#!/bin/bash

#########################################
# Axigen Log Analyzer v1.2              #
# Author: Ciprian Negrila               #
# e-mail: ciprian.negrila@axigen.com    #
# Version: 1.2.3                        #
# Copyright GeCAD Technologies (c) 2007 #
# Axigen division                       #
#########################################
#                                       #
# Exit codes:                           #
#                                       #
#   0 = clean exit                      #
#   1 = script was not run by root      #
#   2 = directory access error          #
#   3 = file access error               #
#   4 = wrong options                   #
# 127 = script had nothing to do        #
#                                       #
#########################################

#Set script version here
VER="1.2"

#Check if we are called by the root user
if [ `id -u` != "0" ]; then
    echo ""
    echo "ERROR: This script has to be run by root."
    echo ""    
    exit 1
fi

#Prepare the environment
echo "Checking if the environment is sane..."

save="/var/log/axi-parser"
data="data"

if [ -e $data/daemon.cnt ]
    then
	echo "File $data/daemon.cnt exists and is accessible."
    else
	echo "Could not access file: $data/daemon.cnt"
	echo "Please make sure the file exists."
        exit 3
fi

if [ -e $data/database.cnt ]
    then
	echo "File $data/database.cnt exists and is accessible."
    else
	echo "Could not access file: $data/database.cnt"
	echo "Please make sure the file exists."
        exit 3
fi

if [ -e $data/services.cnt ]
    then
	echo "File $data/services.cnt exists and is accessible."
    else
	echo "Could not access file: $data/services.cnt"
	echo "Please make sure the file exists."
        exit 3
fi

if [ -d $save/maillog ]
    then
	echo "Directory $save/maillog exists and is accessible."
    else
	mkdir -p $save/maillog
	if [[ "$?" != "0" ]]
	    then echo "Could not create directory: $save/maillog"
		    exit 2
	fi
	echo "Directory $save/maillog has been created."
fi

if [ -d $save/parser/sessions ]
    then
	echo "Directory $save/parser/sessions exists and is accessible."
    else
	mkdir -p $save/parser/sessions
	if [[ "$?" != "0" ]]
	    then echo "Could not create directory: $save/parser/sessions"
		    exit 2
	fi
	echo "Directory $save/parser/sessions has been created."
fi

if [ -d $save/tracer/sessions ]
    then
	echo "Directory $save/tracer/sessions exists and is accessible."
    else
	mkdir -p $save/tracer/sessions
	if [[ "$?" != "0" ]]
	    then echo "Could not create directory: $save/tracer/sessions"
		    exit 2
	fi
	echo "Directory $save/tracer/sessions has been created."
fi

if [ -d $save/splitter ]
    then
	echo "Directory $save/splitter exists and is accessible."
    else
	mkdir -p $save/splitter
	if [[ "$?" != "0" ]]
	    then echo "Could not create directory: $save/splitter"
		    exit 2
	fi
	echo "Directory $save/splitter has been created."
fi

echo "Environment check complete."
echo ""

#A function that displays the usage
usage () {
    echo ""
    echo "= AXIGEN Log Parser version $VER ="
    echo ""
    echo "This is a tool to split, scan and analyze Axigen log files for errors and other information"
    echo "needed to identify specific sessions or problems."
    echo ""
    echo "USAGE: $0 <action> [options]"
    echo ""
    echo "Actions supported are:"
    echo "	parse 	[file]              | Parses an Axigen log file and reports errors found in it."
    echo "	maillog [file]              | Parses a system mail log to find errors."
    echo "	split 	[file]              | Split a log file into unique sessions and store them."
    echo "	trace 	[file] [pattern]    | Trace all sessions containing [pattern] and make a report."
    echo "	clean                       | Cleans up any files generated previously."
    echo ""
    echo ""
    echo "Copyright (c) 2007, GeCAD Technologies"
    echo "Axigen division, support@axigen.com, http://www.axigen.com"
    echo ""
    
}

clean () {

###########################################
# This function only cleans up files left #
# over after any operation.               #
###########################################
echo ""
echo "Please wait, cleaning up auto-generated files and folders..."
echo ""
echo "Removing contents of the \"parser\" folder..."
rm -rf $save/parser/*.txt
if [[ "$?" != "0" ]]
    then echo "WARNING: Could not clean up parser files."
fi	
rm -rf $save/parser/*.id
if [[ "$?" != "0" ]]
    then echo "WARNING: Could not clean up parser files."
fi
rm -rf $save/parser/sessions/*
if [[ "$?" != "0" ]]
    then echo "WARNING: Could not clean up parser files."
fi
echo "Removing contents of the \"tracer\" folder..."
rm -rf $save/tracer/*
if [[ "$?" != "0" ]]
    then echo "WARNING: Could not clean up tracer files."
fi
echo "Removing contents of the \"splitter\" folder..."
rm -rf $save/splitter/*
if [[ "$?" != "0" ]]
    then echo "WARNING: Could not clean up splitter files."
fi
echo "Removing old split log files..."
rm -rf $save/*.txt
if [[ "$?" != "0" ]]
    then echo "WARNING: Could not clean up split log files."
fi
echo ""
echo "Done cleaning up."
echo ""
exit 0

}

parse () {

####################################################################
# This function parses a "default.txt" log file to isolate errors. #
####################################################################

#Testing if we were called correctly and if the log file exists
if [ "$#" != "1" ]; then
    usage
    exit 4
fi

if [ ! -e $1 ]; then
    echo ""
    echo "The log file \"$1\" can't be read. Please make sure the file exists and that you can access it."
    exit 3
fi
echo ""
echo "***Starting the parsing algorithm for the file:" $1
echo "***Please wait."

#We split the log file into service based log files
echo "***Grouping log files on a service basis..."
while read SERVICE
    do
    NR=`cat $1 | grep -c "$SERVICE"`
    if [ "$NR" -gt "0" ]
    then
        cat $1 | grep $SERVICE > "$save/$SERVICE-log.txt"
    fi
    done < $data/services.cnt
echo "***Done splitting log file..."
echo ""
#Search for errors in the files
echo "***Searching for errors now..."
echo ""
ERR="0"
while read MATCH
    do
        COUNT=`cat $1 | grep -c "$MATCH"`
        if [ "$COUNT" -gt "0" ]
                then
                cat $1 | grep "$MATCH" > "$save/parser/log-$MATCH.txt"
                cat $1 | grep "$MATCH" | cut -d: -f 4 >> $save/parser/tmp.id
                echo "***Found $COUNT line(s) that match the \"$MATCH\" error."
                ERR="1"
        fi
    done < "$data/database.cnt"
#Removing duplcate session IDs
if [ "$ERR" = "1" ]
then
    cat "$save/parser/tmp.id" | {
        awk '!x[$0]++'
    } > "$save/parser/sessions.id"
    rm -rf $save/parser/tmp.id
    SESSNR=`wc -l $save/parser/sessions.id | cut -d" " -f 1`
    #Saving unique sessions that have errors...
    echo ""
    echo "***Found $SESSNR unique sessions with errors. Saving all of them..."
    echo ""
    while read ID
        do
        cat "$1" | grep "$ID" > "$save/parser/sessions/$ID-log.txt"
    done < "$save/parser/sessions.id"
    #We don't need the sessions file anymore
    rm -rf  $save/parser/sessions.id
    else
    echo "***No errors found in $1"
    exit 127
fi
echo ""
echo "***Parsing finished for file: $1"
echo "***Results are available in the folder: $save/parser/"
echo ""
exit 0
}

trace () {

########################################################
# This function searches for a pattern in the log file #
# and saves the whole session output in a separate file#
# for investigation.                                   #
########################################################

#Testing if we were called correctly and if the log file exists
if [ "$#" != "2" ]; then
    usage
    exit 4
fi

if [ ! -e $1 ]; then
    echo ""
    echo "The log file <" $1 "> can't be read. Please make sure the file exists and that you can access it."
    exit 3
fi

#Check if a pattern has been provided
if [ -n "$2" ]; then
    echo "Starting the search for \"$2\" in the $1 logfile."
    else
	echo "ERROR: You did not provide a pattern to search for or the pattern is not a text string."
	echo ""
	exit 4
fi

#Check to see if we have anything to trace
COUNT=`cat $1 | grep -ic $2`
if [ "$COUNT" = "0" ]; then
    echo ""
    echo "The pattern you entered was not found (character case ignored). Please use another pattern or a different log file."
    echo ""
    exit 127
fi

#Saving all results for pattern in a file
cat "$1" | grep -i "$2" > "$save/tracer/trace-$2.log"

#Saving all session IDs in a temporary file
cat "$1" | grep -i "$2" | cut -d: -f 4 > "$save/tracer/tmp.id"

#Removing duplicate session IDs from the temp file
cat $save/tracer/tmp.id | {
awk '!x[$0]++'
} > "$save/tracer/trace-$2.id"
rm -rf $save/tracer/tmp.id
SESSNR=`wc -l $save/tracer/trace-$2.id | cut -d" " -f 1`
echo "Found $SESSNR unique sessions for pattern \"$2\". Saving..."

#Start saving entire sessions in separate files
while read SESSION
do
cat "$1" | grep "$SESSION" > "$save/tracer/sessions/session-$SESSION.log"
done < "$save/tracer/trace-$2.id"
echo ""
echo "Sessions matching $2 saved to $save/tracer/sessions/."
echo ""
exit 0
}

maillog () {

#########################################################
#This function looks for errors in a system maillog file#
#########################################################

#Testing if we were called correctly and if the log file exists
if [ "$#" != "1" ]; then
    usage
    exit 4
fi
#Testing if we can access the file in question
if [ ! -e $1 ]; then
    echo ""
    echo "The log file \"$1\" can't be read. Please make sure the file exists and that you can access it."
    exit 3
fi
echo ""
echo "Starting the parsing algorithm for the file:" $1
echo "Please wait..."
echo ""
sleep 0.5

#Searching for errors in the maillog file
echo "Starting to parse log file for errors..."
echo ""
ERR="0"
while read DAEMON
do
    COUNT=`cat $1 | grep -c "$DAEMON"`
    if [ "$COUNT" -gt "0" ]
        then
        cat $1 | grep "$DAEMON" > $save/maillog/"$DAEMON-log.txt"
        echo "Found $COUNT time(s) the \"$DAEMON\" error in the system mail log."
        ERR="1"
    fi
done < $data/daemon.cnt
#Check for errors
if [ "$ERR" = "0" ]
    then
    echo "No errors found in the system mail log file."
fi
echo ""
echo "Done scanning for errors in the system mail log file."
echo "Results saved to $save/maillog/*                      "
echo ""
exit 0

}

split () {

####################################################################
# This function splits a log file into services to isolate errors. #
####################################################################

#Testing if we were called correctly and if the log file exists

if [ "$#" != "1" ]; then
    usage
    exit 4
fi

if [ ! -e $1 ]; then
    echo ""
    echo "The log file \"$1\" can't be read. Please make sure the file exists and that you can access it."
    exit 3
fi
echo ""
echo "***Starting the splitting algorithm for the file:" $1
echo "***Please wait."

#We split the log file into service based log files
echo "***Grouping log files on a service basis..."
while read SERVICE
    do
    NR=`cat $1 | grep -c "$SERVICE"`
    if [ "$NR" -gt "0" ]
    then
        cat $1 | grep $SERVICE > "$save/splitter/$SERVICE-log.txt"
        mkdir "$save/splitter/$SERVICE"
        cat $1 | grep $SERVICE | cut -d: -f 4 >> "$save/splitter/$SERVICE/tmp.id"
        cat $save/splitter/$SERVICE/tmp.id | {
        awk '!x[$0]++'
        } > $save/splitter/$SERVICE/uniqueIDs.txt

        rm -rf $save/splitter/$SERVICE/tmp.id

        while read ID
        do
            echo "*Saving session $ID..."
            cat $1 | grep $ID > "$save/splitter/$SERVICE/$ID-log.txt"
        done < $save/splitter/$SERVICE/uniqueIDs.txt

    fi
    done < $data/services.cnt

echo "***Done splitting log file..."
echo ""
echo "***Finished splitting the file: $1"
echo "***Results are available in the folder: $save/splitter/"
echo ""
exit 0

}

case "$1" in

parse)
    parse "$2"
;;

trace)
    trace "$2" "$3"
;;

maillog)
    maillog "$2"
;;

clean)
    clean
;;

split)
    split "$2"
;;

*)
    usage
;;
esac
